# -*- coding: utf-8 -*-
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class DjeymConfig(AppConfig):
    name = 'djeym'
    verbose_name = _('Yandex maps')



# -*- coding: utf-8 -*-
from functools import wraps

from django.http import HttpResponseForbidden, JsonResponse


def ajax_login_required(view_func):
    """
    Ajax.
    Class-based views - dispatch.
    Verify that the current user is authenticated.
    """
    @wraps(view_func)
    def wrapper(self, request, *args, **kwargs):
        if not request.is_ajax():
            return HttpResponseForbidden()
        if request.user.is_authenticated:
            return view_func(self, request, *args, **kwargs)
        msg = {'detail': 'HTTP 403 Forbidden'}
        return JsonResponse(msg, status=403)
    return wrapper


def ajax_login_required_and_staff(view_func):
    """
    Ajax.
    Class-based views - dispatch.
    Verify that the current user is authenticated.
    """
    @wraps(view_func)
    def wrapper(self, request, *args, **kwargs):
        user = request.user
        if not request.is_ajax():
            return HttpResponseForbidden()
        if user.is_authenticated and user.is_staff:
            return view_func(self, request, *args, **kwargs)
        msg = {'detail': 'HTTP 403 Forbidden'}
        return JsonResponse(msg, status=403)
    return wrapper



# -*- coding: utf-8 -*-

from django import template
from django.conf import settings

from ..models import Map
from ..views import vue_vendors_css_js

register = template.Library()


@register.inclusion_tag('djeym/includes/ymaps_front.html')
def djeym_yandex_map(slug, lang='en'):
    """Load the map to the front page."""

    ymap = Map.objects.filter(slug=slug, active=True).first()
    ctx = {'ymap': ymap}
    if ymap is not None:
        general_settings = ymap.general_settings
        ctx['lang'] = lang or 'en',
        ctx['load_indicator'] = ymap.load_indicator
        ctx['load_indicator_size'] = ymap.load_indicator_size
        ctx['is_heatmap'] = ymap.heatmap_settings.active
        ctx['is_round_theme'] = general_settings.roundtheme
        ctx['width_map_front'] = general_settings.width_map_front
        ctx['height_map_front'] = general_settings.height_map_front
        ctx['presets'] = ymap.presets.values_list('js', flat=True)
        vue_vendors = vue_vendors_css_js('front')
        ctx.update(vue_vendors)
    return ctx


@register.inclusion_tag('djeym/includes/api_ymaps.html')
def djeym_load_api_ymaps(lang='en', ns='djeymYMaps'):
    """Get URL for API Yandex Maps"""

    api_version = '2.1'
    api_key = getattr(settings, 'DJEYM_YMAPS_API_KEY', "")
    is_enterprise = getattr(
        settings, 'DJEYM_YMAPS_API_KEY_FOR_ENTERPRISE', False)
    mode = getattr(settings, 'DJEYM_YMAPS_DOWNLOAD_MODE', 'release')
    lang = lang[:2].lower() if bool(lang) else 'en'

    if lang == 'ru':
        lang += '_RU'
    elif lang == 'en':
        lang += '_US'
    elif lang == 'uk':
        lang += '_UA'
    elif lang == 'tr':
        lang += '_TR'
    else:
        lang = 'en_US'

    return {
        'api_key': api_key,
        'is_enterprise': is_enterprise,
        'api_version': api_version,
        'lang': lang,
        'mode': mode,
        'ns': ns
    }



# -*- coding: utf-8 -*-
from ckeditor_uploader.widgets import CKEditorUploadingWidget
from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import (BlockedIP, GeneralSettings, HeatmapSettings, HeatPoint,
                     Map, MapControls, MarkerIcon, Placemark, Polygon,
                     Polyline, Preset)
from .widgets import CenterMapWidget, CheckIconOffsetWidget


class GeneralSettingsForm(forms.ModelForm):

    class Meta:
        model = GeneralSettings
        fields = ('ymap', 'clustering_edit', 'clustering_site',
                  'cluster_layout', 'cluster_icon_content',
                  'cluster_icon_content_bg_color', 'cluster_icon_content_txt_color',
                  'controls_color', 'buttons_text_color', 'theme_type', 'roundtheme',
                  'panorama', 'width_panel_editor', 'width_panel_front',
                  'open_panel_front', 'img_bg_panel_front', 'tinting_panel_front',
                  'hide_group_name_panel_front', 'width_map_front', 'height_map_front')


class PresetForm(forms.ModelForm):

    class Meta:
        model = Preset
        fields = ('ymap', 'autoheader', 'autobody', 'autofooter',
                  'placemark', 'polyline', 'polygon', 'position')


class HeatPointForm(forms.ModelForm):

    class Meta:
        model = HeatPoint
        fields = ('ymap', 'title', 'weight', 'coordinates')


class HeatmapSettingsForm(forms.ModelForm):

    class Meta:
        model = HeatmapSettings
        fields = ('ymap', 'radius', 'dissipating', 'opacity', 'intensity',
                  'gradient_color1', 'gradient_color2', 'gradient_color3',
                  'gradient_color4', 'active')


class MapControlsForm(forms.ModelForm):

    class Meta:
        model = MapControls
        fields = ('ymap', 'geolocation', 'search', 'provider',
                  'route', 'traffic', 'typeselector', 'fullscreen',
                  'zoom', 'ruler')


class PlacemarkForm(forms.ModelForm):

    class Meta:
        model = Placemark
        fields = ('ymap', 'category', 'subcategories', 'header',
                  'body', 'footer', 'icon_slug', 'coordinates')


class CustomPlacemarkForm(forms.ModelForm):

    class Meta:
        model = Placemark
        fields = ('ymap', 'category', 'subcategories', 'header',
                  'body', 'footer', 'icon_slug', 'coordinates',
                  'user_email', 'active', 'is_user_marker')


class PolylineForm(forms.ModelForm):

    class Meta:
        model = Polyline
        fields = ('ymap', 'category', 'subcategories', 'header', 'body', 'footer',
                  'stroke_width', 'stroke_color', 'stroke_style',
                  'stroke_opacity', 'coordinates')


class PolygonForm(forms.ModelForm):

    class Meta:
        model = Polygon
        fields = ('ymap', 'category', 'subcategories', 'header', 'body', 'footer',
                  'stroke_width', 'stroke_color', 'stroke_opacity', 'fill_color',
                  'stroke_style', 'fill_opacity', 'coordinates')


class BlockedIPForm(forms.ModelForm):

    class Meta:
        model = BlockedIP
        fields = ('ip',)


class CenterMapForm(forms.ModelForm):

    class Meta:
        model = Map
        fields = '__all__'

    center_map = forms.CharField(
        widget=CenterMapWidget,
        label=_('Center map'),
        required=False
    )


class OffsetMarkerIconForm(forms.ModelForm):

    class Meta:
        model = MarkerIcon
        fields = '__all__'

    check_icon_offset = forms.CharField(
        widget=CheckIconOffsetWidget,
        label=_('Check icon offset'),
        required=False
    )


class CKEditorTextareaForm(forms.Form):

    ckeditor_textarea = forms.CharField(
        widget=CKEditorUploadingWidget(config_name='djeym'),
    )



# -*- coding: utf-8 -*-
import random

from django.conf import settings
from django.core.management import BaseCommand, CommandError
from django.utils.lorem_ipsum import paragraphs

from djeym.models import (CategoryPlacemark, Map, MarkerIcon, Placemark,
                          SubCategoryPlacemark)

# Map name: Test Map
# --count - default=100
# python manage.py addmarker
# python manage.py addmarker --count 1000


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--count', type=int, default=100)

    def handle(self, *args, **options):
        count = options['count']

        if count == 0:
            msg = 'Error - The number of markers can not be zero. Parameter: --count'
            raise CommandError(msg)
        elif count < 0:
            msg = 'Error - The number of markers cannot be a negative number. Parameter: --count'
            raise CommandError(msg)

        slug = 'test-map'
        media_url = getattr(settings, 'MEDIA_URL', None)
        ymap = Map.objects.filter(slug=slug).first()

        if ymap is None:
            msg = 'Create a map with the name - Test Map'
            raise CommandError(msg)
        elif media_url is None:
            msg = 'Add MEDIA_URL in settings.py'
            raise CommandError(msg)

        categories = CategoryPlacemark.objects.all()
        count_categories = categories.count()
        subcategories = SubCategoryPlacemark.objects.all()
        count_subcategories = subcategories.count()
        next_number_marker = Placemark.objects.count() + 1
        text = paragraphs(30, common=False)[0]
        icon_name_list = [icon.slug for icon in MarkerIcon.objects.filter(
            icon_collection=ymap.icon_collection)]
        images = [
            '<p><img alt="" src="{}uploads/2019/12/29/1.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/2.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/3.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/4.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/5.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/6.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/7.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/8.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/9.jpg" style="width:322px;" /></p>'.format(
                media_url),
            '<p><img alt="" src="{}uploads/2019/12/29/10.jpg" style="width:322px;" /></p>'.format(
                media_url),
        ]
        count_images = len(images)

        if count_categories == 0:
            msg = 'Create a category for markers!'
            raise CommandError(msg)

        for num in range(count):
            placemark = Placemark.objects.create(
                ymap=ymap,
                category=categories[random.randrange(count_categories)],
                header='<p>Marker Header - {}</p>'.format(next_number_marker),
                body='{0}<p>{1}</p>'.format(
                    images[random.randrange(count_images)], text[:1000]),
                footer='<p>Footer of Marker - {}</p>'.format(
                    next_number_marker),
                icon_slug=random.choice(icon_name_list),
                coordinates='[{0},{1}]'.format(
                    random.randrange(-84, 76), random.randrange(-170, 170))
            )
            if count_subcategories > 0:
                placemark.subcategories.add(*[subcategories[idx] for idx in random.sample(
                    range(count_subcategories), random.randrange(count_subcategories) + 1)])
            next_number_marker += 1



# coding: utf-8
#   ____         ____    __    __
#  /\  _`\    __/\  _`\ /\ \  /\ \/'\_/`\
#  \ \ \/\ \ /\_\ \ \L\_\ `\`\\/'/\      \
#   \ \ \ \ \\/\ \ \  _\L`\ `\ /'\ \ \__\ \
#    \ \ \_\ \\ \ \ \ \L\ \`\ \ \ \ \ \_/\ \
#     \ \____/_\ \ \ \____/  \ \_\ \ \_\\ \_\
#      \/___//\ \_\ \/___/    \/_/  \/_/ \/_/
#            \ \____/
#             \/___/

DjEYM_VERSION = (2, 1, 19)
PYTHON_VERSION = (3, 6)
DJANGO_VERSION = (2, 0)
VUE_VERSION = (2, 6, 11)
VUETIFY_VERSION = (2, 2, 1)

__title__ = 'DjEYM (django-editor-ymaps)'
__djeym_version__ = '.'.join(map(str, DjEYM_VERSION))
__python_version__ = '.'.join(map(str, PYTHON_VERSION))
__django_version__ = '.'.join(map(str, DJANGO_VERSION))
__vue_version__ = '.'.join(map(str, VUE_VERSION))
__vuetify_version__ = '.'.join(map(str, VUETIFY_VERSION))
__author__ = 'kebasyaty'
__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2014 kebasyaty'

default_app_config = 'djeym.apps.DjeymConfig'



# -*- coding: utf-8 -*-
from django.urls import path
from django.views.i18n import JavaScriptCatalog

from . import views

app_name = 'djeym'
urlpatterns = [
    path('jsi18n/',
         JavaScriptCatalog.as_view(packages=['djeym']),
         name='javascript-catalog'),
    path('ajax-balloon-content/',
         views.AjaxBalloonContent.as_view(),
         name='ajax_balloon_content'),
    path('ajax-upload-placemarks/',
         views.AjaxUploadPlacemarks.as_view(),
         name='ajax_get_geo_objects_placemark'),
    path('ajax-upload-settings-front/',
         views.AjaxUploadSettingsFront.as_view(),
         name='ajax_upload_settings_front'),
    path('ajax-upload-heat-points/',
         views.AjaxUploadHeatPoints.as_view(),
         name='ajax_get_heat_points'),
    path('ajax-upload-polylines/',
         views.AjaxUploadPolylines.as_view(),
         name='ajax_get_geo_objects_polyline'),
    path('ajax-upload-polygons/',
         views.AjaxUploadPolygons.as_view(),
         name='ajax_get_geo_objects_polygon'),
    path('ajax-save-geo-object/',
         views.AjaxSaveGeoObjects.as_view(),
         name='ajax_save_geo_object'),
    path('ymeditor/<slug:slug>/',
         views.YMapEditor.as_view(),
         name='ymap_editor'),
    path('ajax-upload-settings-editor/',
         views.AjaxUploadSettingsEditor.as_view(),
         name='ajax_upload_settings_editor'),
    path('ajax-save-cusotm-marker/',
         views.AjaxSaveCusotmMarker.as_view(),
         name='ajax_save_cusotm_marker'),
    path('ajax-cluster-icon/',
         views.AjaxClusterIcon.as_view(),
         name='ajax_cluster_icon'),
    path('ajax-collection-example-icon/',
         views.AjaxCollectionExampleIcon.as_view(),
         name='ajax_collection_example_icon'),
    path('ajax-import-icon-collection/',
         views.AjaxImportIconCollection.as_view(),
         name='ajax_import_icon_collection'),
    path('export-icon-collection/<slug:slug>/',
         views.ExportIconCollection.as_view(),
         name='export_icon_collection'),
    path('ajax-import-tile-source/',
         views.AjaxImportTileSource.as_view(),
         name='ajax_import_tile_source'),
    path('export-tile-source/',
         views.ExportTileSource.as_view(),
         name='export_tile_source'),
    path('ajax-tile-screenshot/',
         views.AjaxTileScreenshot.as_view(),
         name='ajax_tile_screenshot'),
    path('ajax-update-tile-source/',
         views.AjaxUpdateTileSource.as_view(),
         name='ajax_update_tile_source'),
    path('ajax-update-map-controls/',
         views.AjaxUpdateMapControls.as_view(),
         name='ajax_update_map_controls'),
    path('ajax-update-heatmap-settings/',
         views.AjaxUpdateHeatmapSettings.as_view(),
         name='ajax_update-heatmap_settings'),
    path('ajax-activate-heatmap/',
         views.AjaxActivateHeatmap.as_view(),
         name='ajax_activate_heatmap'),
    path('ajax-heatmap-undo-settings/',
         views.AjaxHeatmapUndoSettings.as_view(),
         name='ajax_heatmap_undo_settings'),
    path('ajax-update-likes/',
         views.AjaxUpdateLikes.as_view(),
         name='ajax_update_likes'),
    path('ajax-update-preset-settings/',
         views.AjaxUpdatePresetSettings.as_view(),
         name='ajax_update_preset_settings'),
    path('ajax-update-general-settings/',
         views.AjaxUpdateGeneralSettings.as_view(),
         name='ajax_update_general_settings'),
    path('ajax-load-indicator-icon/',
         views.AjaxLoadIndicatorIcon.as_view(),
         name='ajax_load_indicator_icon'),
    path('ajax-update-load-indicator/',
         views.AjaxUpdateLoadIndicator.as_view(),
         name='ajax_update_load_indicator'),
    path('ajax-update-filters-categories/',
         views.AjaxUpdateFiltersCategories.as_view(),
         name='ajax_update_filters_categories'),
    path('ajax-del-img-bg-panel-front/',
         views.AjaxDeleteImgBgPanelFront.as_view(),
         name='ajax_del_img_bg_panel_front'),
    path('ajax-block-ip-address/',
         views.AjaxBlockIPAddress.as_view(),
         name='ajax_block_ip_address'),
    path('ajax-upload-icon-collection/',
         views.AjaxIconCollection.as_view(),
         name='ajax_upload_icon_collection')
]



# -*- coding: utf-8 -*-
import json
import math
import os
import re
import shutil
import uuid
from decimal import Decimal
from io import BytesIO
from pathlib import Path

from django.apps import apps
from django.core.exceptions import ValidationError
from django.db.models.fields.files import FileDescriptor, ImageFileDescriptor
from django.utils.translation import ugettext_lazy as _
from imagekit.models import ImageSpecField
from lxml import etree

from .globals import DJEYM_YMAPS_ICONS_MAX_SIZE

# SIZE CORRECTION ----------------------------------------------------------------------------------


def get_size_correction(width, height):
    """Get size correction of image."""
    size_width, size_height = Decimal(width), Decimal(height)

    if size_width > DJEYM_YMAPS_ICONS_MAX_SIZE or size_height > DJEYM_YMAPS_ICONS_MAX_SIZE:
        size_width = math.ceil(
            (size_width * (DJEYM_YMAPS_ICONS_MAX_SIZE / size_height)))
        size_height = DJEYM_YMAPS_ICONS_MAX_SIZE
    return [size_width, size_height]


def get_size_from_svg(svg):
    """Get width and height from SVG file."""
    with open(svg.path, mode="rb") as svg_file:
        xml_content = svg_file.read()
        root = etree.XML(xml_content)
        result = {
            "width": int(Decimal(root.get('width'))),
            "height": int(Decimal(root.get('height')))
        }
    return result


# VALIDATORS ---------------------------------------------------------------------------------------

def validate_image(image):
    extension_list = ['.jpg', '.jpeg', '.png']
    size = image.size
    extension = Path(image.name).suffix.lower()

    if extension not in extension_list:
        raise ValidationError(_('Only JPG or PNG format files.'))
    elif not size:
        raise ValidationError(_('Image cannot be 0.0 mb.'))
    elif not size or size > 524288:
        raise ValidationError(_('Maximum image size 0.5 mb.'))


def validate_svg(svg):
    extension_list = ('.svg',)
    extension = Path(svg.name).suffix.lower()

    if extension not in extension_list:
        raise ValidationError(_('Only SVG (file_name.svg) files.'))

    if hasattr(svg, 'temporary_file_path'):
        svg_path = svg.temporary_file_path()
    else:
        if hasattr(svg, 'read'):
            svg_path = BytesIO(svg.read())
        else:
            svg_path = BytesIO(svg['content'])

    if bool(svg_path):
        xml_content = svg_path.read()
        try:
            root = etree.XML(xml_content)
        except etree.XMLSyntaxError:
            svg_path.close()
            raise ValidationError(_('This is an invalid SVG file. Open this file in '
                                    'the vector editor and try to fix it.'))
        tag = root.tag
        if bool(tag) and tag == "{http://www.w3.org/2000/svg}svg":
            if root.get('width') is not None and root.get('height') is not None:
                if root.get('viewBox') is None:
                    svg_path.close()
                    raise ValidationError(_('Missing viewBox attribute'))
            else:
                svg_path.close()
                raise ValidationError(
                    _('There are no width and height attributes in the SVG tag.'))
        else:
            svg_path.close()
            raise ValidationError(_('This is an invalid SVG file. Open this file in '
                                    'the vector editor and try to fix it.'))
    else:
        svg_path.close()
        raise ValidationError(
            _('Unable to read file attributes. The file may be damaged.'))


def validate_coordinates(coordinate):
    try:
        float(coordinate)
    except ValueError:
        raise ValidationError(
            _('To determine the coordinates a numeric value is required.'))


def validate_transparency(coordinate):
    try:
        float(coordinate)
    except ValueError:
        raise ValidationError(
            _('To determine the transparency a numeric value is required.'))


def get_errors_form(*args):
    err_dict = {}
    detail = ''

    for form in args:
        for item in list(form):
            if item.errors:
                err_dict[item.name] = True
                detail += u'{0}: {1}<br>'.format(item.label, item.errors)
            else:
                err_dict[item.name] = False

    return {'err_dict': err_dict, 'detail': detail}


# GET FILENAME -------------------------------------------------------------------------------------

def make_upload_path(instance, filename):
    extension = Path(filename).suffix
    return os.path.join(instance.upload_dir, '{0}{1}'.format(uuid.uuid4(), extension))


def get_filename(filename):
    """CKEditor - Filename Generator"""
    return re.sub(r'\s+', '_', filename).lower()


# CLEANING ORPHAN FILES ----------------------------------------------------------------------------

def cleaning_files_pre_save(sender, instance, **kwargs):
    """Remove old files"""
    old_status = sender.objects.filter(pk=instance.pk)
    dir_name_list = dir(sender)

    if old_status.count() > 0:
        for dir_name in dir_name_list:
            field = getattr(sender, dir_name, None)

            if bool(field) and (isinstance(field, FileDescriptor) or
                                isinstance(field, ImageFileDescriptor) or
                                isinstance(field, ImageSpecField)):

                old_field = getattr(old_status[0], dir_name, None)
                updated_field = getattr(instance, dir_name, None)

                if getattr(old_field, 'name', None) != \
                        getattr(updated_field, 'name', None):
                    try:
                        path = getattr(old_field, 'path', None)

                        if bool(path) and os.path.exists(path):
                            if isinstance(field, ImageSpecField):
                                pattern = re.compile(
                                    r'(/[-_\w()]+\.[a-zA-Z]{3,4}$)')
                                shutil.rmtree(pattern.sub('', path))
                            else:
                                os.remove(path)
                    except (FileNotFoundError, ValueError, TypeError):
                        pass


def cleaning_files_pre_delete(sender, instance, **kwargs):
    """Delete orphan files"""
    dir_name_list = dir(sender)

    for dir_name in dir_name_list:
        field = getattr(sender, dir_name, None)

        if bool(field) and (isinstance(field, FileDescriptor) or
                            isinstance(field, ImageFileDescriptor) or
                            isinstance(field, ImageSpecField)):

            target_field = getattr(instance, dir_name, None)

            try:
                path = getattr(target_field, 'path', None)

                if bool(path) and os.path.exists(path):
                    if isinstance(field, ImageSpecField):
                        pattern = re.compile(
                            r'(/[-_\w()]+\.[a-zA-Z]{3,4}$)')
                        shutil.rmtree(pattern.sub('', path))
                    else:
                        os.remove(path)
            except (FileNotFoundError, ValueError, TypeError):
                pass


# REFRESH JSON-CODE --------------------------------------------------------------------------------

def placemark_update_json_code(instance):
    """Refresh json code for Placemark"""
    json_code = json.loads(instance.json_code)

    if json_code["id"] == 0:
        json_code["id"] = instance.pk

    json_code['geometry']['coordinates'] = json.loads(instance.coordinates)
    json_code["properties"]["id"] = instance.pk
    json_code["properties"]["categoryID"] = instance.category.pk
    json_code["properties"]["subCategoryIDs"] = list(
        instance.subcategories.all().values_list('pk', flat=True))
    json_code["properties"]["iconSlug"] = instance.icon_slug

    if instance.icon_slug != 'djeym-marker-default':
        icon_collection = instance.ymap.icon_collection
        icon_marker = apps.get_model('djeym', 'MarkerIcon').objects.get(
            icon_collection=icon_collection, slug=instance.icon_slug)
        json_code["options"]['iconImageHref'] = icon_marker.svg.url
        json_code["options"]['iconImageSize'] = json.loads(
            icon_marker.get_size())
        json_code["options"]['iconImageOffset'] = json.loads(
            icon_marker.get_offset())
    else:
        json_code["options"]['iconImageHref'] = '/static/djeym/img/center.svg'
        json_code["options"]['iconImageSize'] = [32, 60]
        json_code["options"]['iconImageOffset'] = [-16, -60]
    return json.dumps(json_code, ensure_ascii=False)


def polyline_update_json_code(instance):
    """Refresh json code for Polyline"""
    json_code = json.loads(instance.json_code)

    if json_code["id"] == 0:
        json_code["id"] = instance.pk

    json_code['geometry']['coordinates'] = json.loads(instance.coordinates)
    json_code["properties"]["id"] = instance.pk
    json_code["properties"]["categoryID"] = instance.category.pk
    json_code["properties"]["subCategoryIDs"] = list(
        instance.subcategories.all().values_list('pk', flat=True))
    json_code["options"]["strokeWidth"] = float(instance.stroke_width)
    json_code["options"]["strokeColor"] = instance.stroke_color
    json_code["options"]["strokeStyle"] = instance.stroke_style
    json_code["options"]["strokeOpacity"] = float(instance.stroke_opacity)
    return json.dumps(json_code, ensure_ascii=False)


def polygon_update_json_code(instance):
    """Refresh json code for Polygon"""
    json_code = json.loads(instance.json_code)

    if json_code["id"] == 0:
        json_code["id"] = instance.pk

    json_code['geometry']['coordinates'] = json.loads(instance.coordinates)
    json_code["properties"]["id"] = instance.pk
    json_code["properties"]["categoryID"] = instance.category.pk
    json_code["properties"]["subCategoryIDs"] = list(
        instance.subcategories.all().values_list('pk', flat=True))
    json_code["options"]["strokeWidth"] = float(instance.stroke_width)
    json_code["options"]["strokeColor"] = instance.stroke_color
    json_code["options"]["strokeStyle"] = instance.stroke_style
    json_code["options"]["strokeOpacity"] = float(instance.stroke_opacity)
    json_code["options"]["fillColor"] = instance.fill_color
    json_code["options"]["fillOpacity"] = float(instance.fill_opacity)
    return json.dumps(json_code, ensure_ascii=False)


def heatpoint_update_json_code(instance):
    """Refresh json code for HeatPoint"""
    json_code = json.loads(instance.json_code)

    if json_code["id"] == 0:
        json_code["id"] = instance.id

    json_code['geometry']['coordinates'] = json.loads(instance.coordinates)
    json_code["properties"]["weight"] = int(instance.weight)
    return json.dumps(json_code, ensure_ascii=False)
